# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.apps import apps
from django.contrib.contenttypes.management import update_contenttypes
from django.db import migrations

from wooey.settings import get as get_setting


def update_all_contenttypes(**kwargs):
    # from http://stackoverflow.com/questions/29550102/importerror-cannot-import-name-update-all-contenttypes
    for app_config in apps.get_app_configs():
        update_contenttypes(app_config, **kwargs)

def gen_userfiles(apps, schema_editor):
    WooeyFile = apps.get_model('wooey', 'WooeyFile')
    UserFile = apps.get_model('wooey', 'UserFile')
    Favorite = apps.get_model('wooey', 'Favorite')
    update_all_contenttypes()
    ContentType = apps.get_model("contenttypes", "ContentType")
    ctype = ContentType.objects.get(model='wooeyfile')
    new_ctype = ContentType.objects.get(model='userfile')
    import os
    checksums = {}
    to_delete = []
    for obj in WooeyFile.objects.all():
        # we only do this for uploads
        if obj.parameter is None or obj.parameter.parameter.is_output == True:
            file_to_use = obj
        else:
            checksum = obj.checksum
            file_to_use = checksums.get(checksum, obj)
            if checksum is not None and checksum not in checksums:
                checksums[checksum] = file_to_use
            if file_to_use != obj:
                to_delete.append(obj.pk)
        user_file = UserFile(filename=os.path.split(obj.filepath.name)[1], job=obj.job,
                             parameter=obj.parameter, system_file=file_to_use)
        user_file.save()
        favorites = Favorite.objects.filter(content_type=ctype, object_id=obj.id)
        for favorite in favorites:
            favorite.content_object = user_file
            favorite.content_type = new_ctype
            favorite.save()
    WooeyFile.objects.filter(pk__in=to_delete).delete()

def setup_wooey_files(apps, schema_editor):
    from six.moves import StringIO
    from django.core.files import File
    from wooey.backend.utils import get_storage

    storage = get_storage()

    WooeyFile = apps.get_model('wooey', 'WooeyFile')
    Favorite = apps.get_model('wooey', 'Favorite')
    WooeyJob = apps.get_model('wooey', 'WooeyJob')
    ScriptParameter = apps.get_model('wooey', 'ScriptParameter')
    ScriptParameters = apps.get_model('wooey', 'ScriptParameters')
    ScriptParameterGroup = apps.get_model('wooey', 'ScriptParameterGroup')
    ScriptVersion = apps.get_model('wooey', 'ScriptVersion')
    Script = apps.get_model('wooey', 'Script')
    User = apps.get_model('auth', 'User')
    update_all_contenttypes()
    ContentType = apps.get_model("contenttypes", "ContentType")
    ctype = ContentType.objects.get(model='wooeyfile')

    user = User.objects.create(username='test user')

    script = Script.objects.create(
        script_name='Test'
    )

    script_version = ScriptVersion.objects.create(
        script=script,
        script_path=get_storage().save('fake_script', File(StringIO('nonsense'))),
    )

    script_parameter_group = ScriptParameterGroup.objects.create(
        group_name='blahh',
        script_version=script_version,
    )

    script_parameter = ScriptParameter.objects.create(
        script_version=script_version,
        short_param='blah',
        script_param='--blah',
        form_field='FileField',
        input_type='file',
        parameter_group=script_parameter_group,
        is_output=False,
    )

    job = WooeyJob.objects.create(
        script_version=script_version,
        job_name='job1',
    )

    job2 = WooeyJob.objects.create(
        script_version=script_version,
        job_name='job2',
    )

    # make wooey files
    buffer = StringIO('file1')
    file1 = get_storage().save('file1', File(buffer))
    file2 = get_storage().save('file1', File(buffer))

    script_parameters = ScriptParameters.objects.create(
        parameter=script_parameter,
        job=job,
        _value=file1,
    )

    script_parameters2 = ScriptParameters.objects.create(
        parameter=script_parameter,
        job=job2,
        _value=file2,
    )


    wooey_file1 = WooeyFile.objects.create(
        filepath=file1,
        job=job,
        parameter=script_parameters,
        checksum='abc123',
    )

    wooey_file1_copy = WooeyFile.objects.create(
        filepath=file2,
        job=job2,
        parameter=script_parameters2,
        checksum='abc123',
    )

    # make the second a favorite file
    Favorite.objects.create(
        content_type=ctype,
        object_id=wooey_file1_copy.pk,
        user=user
    )



def confirm_data_migration(apps, schema_editor):
    WooeyFile = apps.get_model('wooey', 'WooeyFile')
    UserFile = apps.get_model('wooey', 'UserFile')
    Favorite = apps.get_model('wooey', 'Favorite')
    WooeyJob = apps.get_model('wooey', 'WooeyJob')
    ContentType = apps.get_model("contenttypes", "ContentType")
    userfile_type = ContentType.objects.get(model='userfile')

    # We should have one wooeyfile and 2 userfiles
    wooeyfiles = list(WooeyFile.objects.all())
    userfiles = list(UserFile.objects.all())
    assert len(wooeyfiles) == 1
    assert len(userfiles) == 2

    # Ensure favorites now point at UserFiles as well
    favorite = Favorite.objects.first()
    assert favorite.content_type_id == userfile_type.id
    assert favorite.object_id in [i.pk for i in userfiles]

    # Ensure both wooey jobs point at their respective userfiles
    for wooeyjob in WooeyJob.objects.all():
        # There is one one file per wooey job
        assert wooeyjob.userfile_set.first().filename is not None

    # Assert the userfiles all point at the same wooeyfile
    wooeyfile = WooeyFile.objects.first()
    for userfile in UserFile.objects.all():
        assert userfile.system_file_id == wooeyfile.id

def cleanup_tests(apps, schema_editor):
    WooeyFile = apps.get_model('wooey', 'WooeyFile')
    Favorite = apps.get_model('wooey', 'Favorite')
    WooeyJob = apps.get_model('wooey', 'WooeyJob')
    ScriptParameter = apps.get_model('wooey', 'ScriptParameter')
    ScriptParameters = apps.get_model('wooey', 'ScriptParameters')
    ScriptParameterGroup = apps.get_model('wooey', 'ScriptParameterGroup')
    ScriptVersion = apps.get_model('wooey', 'ScriptVersion')
    Script = apps.get_model('wooey', 'Script')
    WooeyFile.objects.all().delete()
    Favorite.objects.all().delete()
    WooeyJob.objects.all().delete()
    ScriptParameter.objects.all().delete()
    ScriptParameters.objects.all().delete()
    ScriptParameterGroup.objects.all().delete()
    ScriptVersion.objects.all().delete()
    Script.objects.all().delete()

class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0018_userfile'),
    ]

    operations = []

    if get_setting('TESTING', False):
        operations.append(migrations.RunPython(setup_wooey_files))

    operations.append(
        migrations.RunPython(gen_userfiles),
    )

    if get_setting('TESTING', False):
        operations.append(migrations.RunPython(confirm_data_migration))
        operations.append(migrations.RunPython(cleanup_tests))



import os

from django.test import TestCase
from django.http.request import MultiValueDict
from django.core.files.uploadedfile import SimpleUploadedFile

from ..backend import utils
from ..forms import WooeyForm
from ..forms import config as forms_config

from . import config
from . import mixins
from ..models import WooeyJob


class FormTestCase(mixins.ScriptFactoryMixin, mixins.FileCleanupMixin, TestCase):

    @staticmethod
    def get_mvdict(data):
        return MultiValueDict(data)

    def test_master_form(self):
        script_version = self.translate_script
        form = utils.get_master_form(script_version=script_version)
        self.assertTrue(isinstance(form, WooeyForm))
        qdict = self.get_mvdict(config.SCRIPT_DATA['translate'].get('data'))
        utils.validate_form(form=form, data=qdict,
                            files=config.SCRIPT_DATA['translate'].get('files'))
        self.assertTrue(form.is_valid())

    def test_group_form(self):
        script_version = self.translate_script
        form = utils.get_form_groups(script_version=script_version)
        self.assertEqual(len(form['groups']), 2)
        # test we can hide parameters and groups
        from wooey.models import ScriptParameterGroup, ScriptParameter
        groups = ScriptParameterGroup.objects.filter(script_version=script_version)
        group = groups[1]
        group.hidden = True
        group.save()
        form = utils.get_form_groups(script_version=script_version)
        self.assertEqual(len(form['groups']), 1, 'Script Parameter group is hidden but shown')
        group.hidden = False
        group.save()
        form = utils.get_form_groups(script_version=script_version)
        self.assertEqual(len(form['groups']), 2, 'Script Parameter group is shown but hidden')
        param = ScriptParameter.objects.get(script_version=script_version, slug='out')
        param.hidden = True
        param.save()
        form = utils.get_form_groups(script_version=script_version)
        self.assertNotIn('out', form['groups'][1]['form'].fields, 'Script Parameter is hidden but shown')
        param.hidden = False
        param.save()
        form = utils.get_form_groups(script_version=script_version)
        self.assertIn('out', form['groups'][1]['form'].fields, 'Script Parameter is shown but hidden')


    def test_multiplechoice_form(self):
        script_version = self.choice_script
        form = utils.get_master_form(script_version=script_version)
        # check our wrapper is in the form render
        form_str = str(form)
        self.assertTrue(forms_config.WOOEY_MULTI_WIDGET_ANCHOR in form_str)
        self.assertTrue(forms_config.WOOEY_MULTI_WIDGET_ATTR in form_str)
        qdict = self.get_mvdict(config.SCRIPT_DATA['choices'].get('data'))
        fdict = config.SCRIPT_DATA['choices'].get('files')
        uploaded_files = {}
        storage = utils.get_storage(local=True)
        for file_key, files in fdict.items():
            uploaded_files[file_key] = []
            for file_location in files:
                file_name = os.path.split(file_location.name)[1]
                uploaded_files[file_key].append(SimpleUploadedFile(file_name, file_location.read()))

        utils.validate_form(form=form, data=qdict,
                            files=uploaded_files)
        self.assertTrue(form.is_valid())
        # test we can create a job from this form
        # this is implemented to put data and files in the same dictionary, so update it
        form.cleaned_data.update(uploaded_files)
        job = utils.create_wooey_job(script_version_pk=script_version.pk, data=form.cleaned_data)
        # check the files are here
        file_param = 'multiple_file_choices'
        files = [i.value for i in job.get_parameters() if i.parameter.slug == file_param]
        self.assertEqual(len(files), len(uploaded_files.get(file_param)))

    def test_without_args_form(self):
        script_version = self.without_args
        form = utils.get_master_form(script_version=script_version)
        # check our wrapper is in the form render
        # in a normal POST request, the data will contain {'wooey_type': script_pk}. This will cause the is_bound
        # attribute of the form to be set. Otherwise, this test will always fail. It is OK to set this data
        # in the test, since it is how it will be recapitulated in the normal use case
        form_data = form.initial
        form_data.update(config.SCRIPT_DATA['without_args'].get('data'))
        utils.validate_form(form=form, data=form_data)
        self.assertTrue(form.is_valid(), form.errors)
        # test we can create a job from this form
        job = utils.create_wooey_job(script_version_pk=script_version.pk, data=form.cleaned_data)
        job.submit_to_celery()
        # get the job again
        job = WooeyJob.objects.get(pk=job.pk)
        self.assertEqual(job.status, WooeyJob.COMPLETED)


from __future__ import absolute_import
from django.views.generic import CreateView
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from django.forms.models import modelform_factory
from django.contrib.auth import login, authenticate, get_user_model
from django.utils.translation import ugettext_lazy as _
from django.utils.encoding import force_text

from .. import settings as wooey_settings
from ..django_compat import JsonResponse


class WooeyRegister(CreateView):
    template_name = 'wooey/registration/register.html'
    model = get_user_model()
    fields = ('username', 'email', 'password')

    def dispatch(self, request, *args, **kwargs):
        if wooey_settings.WOOEY_AUTH is False:
            return HttpResponseRedirect(wooey_settings.WOOEY_REGISTER_URL)
        return super(WooeyRegister, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.object = None
        form = self.get_form_class()
        post = request.POST.copy()
        post['username'] = post['username'].lower()
        form = form(post)
        if request.POST['password'] != request.POST['password2']:
            form.add_error('password', _('Passwords do not match.'))
        if request.POST['username'].lower() == 'admin':
            form.add_error('username', _('Reserved username.'))
        if not request.POST['email']:
            form.add_error('email', _('Please enter your email address.'))
        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)

    def get_success_url(self):
        next_url = self.request.POST.get('next')
        # for some bizarre reason the password isn't setting by the modelform
        self.object.set_password(self.request.POST['password'])
        self.object.save()
        auser = authenticate(username=self.object.username, password=self.request.POST['password'])
        login(self.request, auser)
        return reverse(next_url) if next_url else reverse('wooey:wooey_home')


def wooey_login(request):
    if wooey_settings.WOOEY_AUTH is False:
        return HttpResponseRedirect(wooey_settings.WOOEY_LOGIN_URL)
    User = get_user_model()
    form = modelform_factory(User, fields=('username', 'password'))
    user = User.objects.filter(username=request.POST.get('username'))
    if user:
        user = user[0]
    else:
        user = None
    form = form(request.POST, instance=user)
    if form.is_valid():
        data = form.cleaned_data
        user = authenticate(username=data['username'], password=data['password'])
        if user is None:
            return JsonResponse({'valid': False, 'errors': {'__all__': [force_text(_('You have entered an invalid username or password.'))]}})
        login(request, user)
        return JsonResponse({'valid': True, 'redirect': request.POST['next']})
    else:
        return JsonResponse({'valid': False, 'errors': form.errors})



from __future__ import absolute_import

from django.db.models.signals import post_delete, pre_save, post_save
from django.db.utils import InterfaceError, DatabaseError
from django import db

from celery.signals import task_postrun, task_prerun, task_revoked


@task_postrun.connect
@task_prerun.connect
def task_completed(sender=None, **kwargs):
    task_kwargs = kwargs.get('kwargs')
    job_id = task_kwargs.get('wooey_job')
    # Just return if it is not a wooey_job!
    if not job_id:
        return

    from .models import WooeyJob
    from celery import states
    try:
        job = WooeyJob.objects.get(pk=job_id)
    except (InterfaceError, DatabaseError) as e:
        db.connection.close()
        job = WooeyJob.objects.get(pk=job_id)
    state = kwargs.get('state')
    if state:
        job.status = WooeyJob.COMPLETED if state == states.SUCCESS else state
    job.celery_id = kwargs.get('task_id')
    job.save()

from .models import ScriptVersion


def skip_script(instance):
    return getattr(instance, '_script_cl_creation', False) or getattr(instance, '_script_upgrade', False) or getattr(instance, '_rename_script', False)


def script_version_presave(instance, **kwargs):
    created = instance.pk is None
    if not created:
        if 'script_path' in instance.changed_fields and not skip_script(instance):
            instance.script_iteration += 1
            instance._script_upgrade = True
            instance.pk = None


def script_version_postsave(instance, created, **kwargs):
    from .backend import utils
    if created and (not skip_script(instance) or getattr(instance, '_script_upgrade', False)):
        res = utils.add_wooey_script(script_version=instance, group=instance.script.script_group)
        instance._script_upgrade = False
        instance._script_cl_creation = False
        instance._rename_script = False
        if res['valid'] is False:
            # delete the model on exceptions.
            # TODO: use django messages backend to propogate this message to the admin
            instance.delete()
            raise BaseException(res['errors'])
    utils.reset_form_factory(script_version=instance)

pre_save.connect(script_version_presave, sender=ScriptVersion)
post_save.connect(script_version_postsave, sender=ScriptVersion)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import wooey.models.mixins


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0017_wooeyfile_generate_checksums'),
    ]

    operations = [
        migrations.CreateModel(
            name='UserFile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('filename', models.TextField()),
                ('job', models.ForeignKey(to='wooey.WooeyJob')),
                ('parameter', models.ForeignKey(blank=True, to='wooey.ScriptParameters', null=True)),
            ],
            bases=(wooey.models.mixins.WooeyPy2Mixin, models.Model),
        ),
        migrations.AddField(
            model_name='userfile',
            name='system_file',
            field=models.ForeignKey(to='wooey.WooeyFile'),
        ),
    ]



from . version import DJANGO_VERSION, DJ18, DJ17, DJ16, DJ19
from django.conf import settings

try:
    settings.configure()
except RuntimeError:
    pass

# JSON compatibility fixes
if DJANGO_VERSION < DJ17:
    import json
    from django.http import HttpResponse


    class JsonResponse(HttpResponse):
        """
            JSON response
            # from https://gist.github.com/philippeowagner/3179eb475fe1795d6515
        """

        def __init__(self, content, mimetype='application/json', status=None, content_type=None, **kwargs):
            super(JsonResponse, self).__init__(
                content=json.dumps(content),
                mimetype=mimetype,
                status=status,
                content_type=content_type,
            )


else:
    from django.http import JsonResponse

if DJANGO_VERSION < DJ19:
    from django.template.base import TagHelperNode, parse_bits
else:
    from django.template.library import TagHelperNode, parse_bits

if DJANGO_VERSION >= DJ18:
    from django.template import Engine
    get_template_from_string = Engine.get_default().from_string
else:
    from django.template.loader import get_template_from_string


if DJANGO_VERSION < DJ17:
    from django.forms.util import flatatt, format_html

else:
    from django.forms.utils import flatatt, format_html

def get_cache(cache):
    if DJANGO_VERSION < DJ17:
        from django.core.cache import get_cache
        return get_cache(cache)
    else:
        from django.core.cache import caches
        return caches[cache]



__author__ = 'chris'
import os
import sys
from django.core.management.base import BaseCommand, CommandError
from django.core.files import File
from django.conf import settings

from ...backend.utils import add_wooey_script, get_storage, default_storage
from ... import settings as wooey_settings


class Command(BaseCommand):
    help = 'Adds a script to Wooey'

    def add_arguments(self, parser):
        parser.add_argument('script', type=str, help='A script or folder of scripts to add to Wooey.')
        parser.add_argument(
            '--group',
            dest='group',
            default='Wooey Scripts',
            help='The name of the group to create scripts under. Default: Wooey Scripts'
        )
        parser.add_argument(
            '--update',
            dest='update',
            action='store_true',
            help='If declared, attempt to match file names to script names and update scripts.'
        )

    def handle(self, *args, **options):
        script = options.get('script')
        if not script:
            if len(args):
                script = args[-1]
            else:
                raise CommandError('You must provide a script path or directory containing scripts.')
        if not os.path.exists(script):
            raise CommandError('{0} does not exist.'.format(script))
        group = options.get('group', 'Wooey Scripts')
        scripts = [os.path.join(script, i) for i in os.listdir(script)] if os.path.isdir(script) else [script]
        converted = 0
        for script in scripts:
            if script.endswith('.pyc') or '__init__' in script:
                continue
            if script.endswith('.py'):
                sys.stdout.write('Converting {}\n'.format(script))
                # copy the script to our storage
                base_name = os.path.splitext(os.path.split(script)[1])[0]
                with open(script, 'r') as f:
                    script = default_storage.save(os.path.join(wooey_settings.WOOEY_SCRIPT_DIR, os.path.split(script)[1]), File(f))
                    if wooey_settings.WOOEY_EPHEMERAL_FILES:
                        # save it locally as well (the default_storage will default to the remote store)
                        local_storage = get_storage(local=True)
                        local_storage.save(os.path.join(wooey_settings.WOOEY_SCRIPT_DIR, os.path.split(script)[1]), File(f))
                add_kwargs = {
                    'script_path': script,
                    'group': group,
                    'script_name': base_name,
                }
                add_script = True
                if options.get('update'):
                    from wooey.models import Script
                    existing_script = Script.objects.filter(script_name=base_name)
                    if len(existing_script) == 1:
                        script_version = existing_script[0].latest_version
                        script_version.script_path = script
                        script_version.default_version = False
                        add_script = False
                        script_version.save()
                        converted += 1
                        # add_kwargs['script_version'] = script_version
                if add_script:
                    res = add_wooey_script(**add_kwargs)
                    if res['valid']:
                        converted += 1
        sys.stdout.write('Converted {} scripts\n'.format(converted))



from __future__ import division, absolute_import
from django import template
from django.utils.safestring import mark_safe
from django.contrib.contenttypes.models import ContentType
from django.template.base import TemplateSyntaxError

from inspect import getargspec
import hashlib

from six.moves.urllib_parse import urlencode

from .. import settings as wooey_settings
from ..django_compat import TagHelperNode, parse_bits
from ..version import DJANGO_VERSION, DJ19


class Library(template.Library):
    if DJANGO_VERSION >= DJ19:
        simple_assignment_tag = template.Library.simple_tag
    else:
        def simple_assignment_tag(self, func=None, takes_context=None, name=None):
            '''
            Like assignment_tag but when "as" not provided, falls back to simple_tag behavior!
            NOTE: this is based on Django's assignment_tag implementation, modified as needed.

            https://gist.github.com/natevw/f14812604be62c073461
            '''
            # (nvw) imports necessary to match original context

            def dec(func):
                params, varargs, varkw, defaults = getargspec(func)

                # (nvw) added from Django's simple_tag implementation
                class SimpleNode(TagHelperNode):
                    def render(self, context):
                        resolved_args, resolved_kwargs = self.get_resolved_arguments(context)
                        return func(*resolved_args, **resolved_kwargs)

                class AssignmentNode(TagHelperNode):
                    def __init__(self, takes_context, args, kwargs, target_var):
                        super(AssignmentNode, self).__init__(takes_context, args, kwargs)
                        self.target_var = target_var

                    def render(self, context):
                        resolved_args, resolved_kwargs = self.get_resolved_arguments(context)
                        context[self.target_var] = func(*resolved_args, **resolved_kwargs)
                        return ''

                function_name = (name or
                    getattr(func, '_decorated_function', func).__name__)

                def compile_func(parser, token):
                    bits = token.split_contents()[1:]
                    if len(bits) > 2 and bits[-2] == 'as':
                        target_var = bits[-1]
                        bits = bits[:-2]
                        args, kwargs = parse_bits(parser, bits, params,
                            varargs, varkw, defaults, takes_context, function_name)
                        return AssignmentNode(takes_context, args, kwargs, target_var)
                    else:
                        args, kwargs = parse_bits(parser, bits, params,
                            varargs, varkw, defaults, takes_context, function_name)
                        return SimpleNode(takes_context, args, kwargs)

                compile_func.__doc__ = func.__doc__
                self.tag(function_name, compile_func)
                return func

            if func is None:
                # @register.assignment_tag(...)
                return dec
            elif callable(func):
                # @register.assignment_tag
                return dec(func)
            else:
                raise TemplateSyntaxError("Invalid arguments provided to assignment_tag")

register = Library()


@register.simple_tag
def get_user_favorite_count(user, app, model):
    from ..models import Favorite
    ctype = ContentType.objects.get(app_label=app, model=model)
    # Return the current total number for UI updates
    favorites_count = Favorite.objects.filter(content_type=ctype, user=user).count()
    return str(favorites_count)


@register.simple_assignment_tag
def get_wooey_setting(name):
    return getattr(wooey_settings, name, "")


@register.filter
def divide(value, arg):
    try:
        return float(value)/float(arg)
    except ZeroDivisionError:
        return None


@register.filter
def endswith(value, arg):
    return str(value).endswith(arg)


@register.filter
def valid_user(obj, user):
    from ..backend import utils
    valid = utils.valid_user(obj, user)
    return True if valid.get('valid') else valid.get('display')


@register.filter
def complete_job(status):
    from ..models import WooeyJob
    from celery import states
    return status in (WooeyJob.COMPLETED, states.REVOKED)


@register.filter
def numericalign(s):
    """
    Takes an input string of "number units" splits it
    and outputs it with each half wrapped in 50% width
    span. Has the effect of centering numbers on the unit part.
    :param s:
    :return: s
    """
    number, units = s.split()
    return mark_safe('<span class="numericalign numericpart">%s</span><span class="numericalign">&nbsp;%s</span>' % (number, units))


@register.filter
def app_model_id(obj):
    """
    Returns a app-model-id string for a given object
    :param obj:
    :return:
    """
    ct = ContentType.objects.get_for_model(obj)

    return '%s-%s-%s' % (ct.app_label, ct.model, obj.id)


@register.filter
def concat(arg1, arg2):
    """concatenate arg1 & arg2"""
    return str(arg1) + str(arg2)


class GravatarUrlNode(template.Node):
    def __init__(self, email, size):
        self.email = template.Variable(email)
        self.size = template.Variable(size)

    def render(self, context):
        try:
            email = self.email.resolve(context)
        except template.VariableDoesNotExist:
            return ''

        try:
            size = self.size.resolve(context)
        except template.VariableDoesNotExist:
            return ''

        url = "http://www.gravatar.com/avatar/" + hashlib.md5(email.lower().encode()).hexdigest() + "?"
        url += urlencode({'s': str(size)})

        return url


@register.tag
def gravatar(parser, token):
    try:
        tag_name, email, size = token.split_contents()

    except ValueError:
        raise template.TemplateSyntaxError("%r tag requires email and size arguments" % token.contents.split()[0])

    return GravatarUrlNode(email, size)


@register.filter
def get_range(value):
    return range(int(value))


@register.simple_assignment_tag(takes_context=True)
def absolute_url(context, url):
    request = context['request']
    return request.build_absolute_uri(url)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-02 14:25
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0022_add_collapse_arguments'),
    ]

    operations = [
        migrations.AddField(
            model_name='scriptparameter',
            name='script_versions',
            field=models.ManyToManyField(related_name='scriptparameters', to='wooey.ScriptVersion'),
        ),
        migrations.AlterField(
            model_name='scriptparameter',
            name='script_version',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='scriptparameter', to='wooey.ScriptVersion'),
        ),
    ]



import sys
import os
import subprocess
import shutil
from argparse import ArgumentParser
from django.template import Context

# This is needed on django 1.9
from .. version import DJANGO_VERSION, DJ19
if DJANGO_VERSION >= DJ19:
    from django.conf import settings
    import django
    settings.configure()
    django.setup()

import wooey

from .. import django_compat


def which(pgm):
    # from http://stackoverflow.com/questions/9877462/is-there-a-python-equivalent-to-the-which-command
    path=os.getenv('PATH')
    for p in path.split(os.path.pathsep):
        p=os.path.join(p, pgm)
        if os.path.exists(p) and os.access(p, os.X_OK):
            return p


def walk_dir(templates, dest, filter=None):
    l = []
    for root, folders, files in os.walk(templates):
        for filename in files:
            if filename.endswith('.pyc') or (filter and filename not in filter):
                continue
            relative_dir = '.{0}'.format(os.path.split(os.path.join(root, filename).replace(templates, ''))[0])
            l.append((os.path.join(root, filename), os.path.join(dest, relative_dir)))
    return l


def bootstrap(env=None, cwd=None):
    if env is None:
        env = os.environ
    parser = ArgumentParser(description="Create a Django project with a Wooey app included.")
    parser.add_argument('-p', '--project', help='The name of the django project to create.', type=str, required=True)
    args = parser.parse_args()

    project_name = args.project
    new_project = not os.path.exists(project_name)
    if not new_project:
        sys.stderr.write('Project {0} already exists.\n'.format(project_name))
        sys.exit(1)
    env['DJANGO_SETTINGS_MODULE'] = ''
    admin_command = [sys.executable] if sys.executable else []
    admin_path = which('django-admin.py')
    if admin_path is None:
        # on windows, we may need to look for django-admin.exe
        platform = sys.platform
        if platform == "win32":
            admin_path = which('django-admin')
            if admin_path is None:
                admin_path = which('django-admin.exe')
                if admin_path is not None:
                    admin_command = []
    if admin_path is None:
        sys.stderr.write('Unable to find django-admin command. Please check your PATH and ensure django-admin is accessible.\n')
        sys.exit(1)
    admin_command.extend([admin_path, 'startproject', project_name])
    admin_kwargs = {'env': env}
    if cwd is not None:
        admin_kwargs.update({'cwd': cwd})
    subprocess.call(admin_command, **admin_kwargs)
    project_root = project_name
    project_base_dir = os.path.join(os.path.realpath(os.path.curdir), project_root, project_name)

    wooify_folder = os.path.split(os.path.realpath(wooey.__file__))[0]
    project_template_dir = os.path.join(wooify_folder, 'conf', 'project_template')

    context = Context(
        dict({
            'project_name': project_name,
        },
        autoescape=False
    ))

    template_files = []

    template_files += walk_dir(project_template_dir, project_base_dir)

    for template_file, dest_dir in template_files:
        template_file = open(template_file)
        content = template_file.read()
        template = django_compat.get_template_from_string(content)
        content = template.render(context)
        content = content.encode('utf-8')
        to_name = os.path.join(dest_dir, os.path.split(template_file.name)[1])
        try:
            os.mkdir(dest_dir)
        except:
            pass
        with open(to_name, 'wb') as new_file:
            new_file.write(content)

    # move the django settings to the settings path so we don't have to chase Django changes.
    shutil.move(os.path.join(project_base_dir, 'settings.py'), os.path.join(project_base_dir, 'settings', 'django_settings.py'))
    # do the same with urls
    shutil.move(os.path.join(project_base_dir, 'urls.py'), os.path.join(project_base_dir, 'urls', 'django_urls.py'))
    env['DJANGO_SETTINGS_MODULE'] = '.'.join([project_name, 'settings', 'user_settings'])
    if django_compat.DJANGO_VERSION >= django_compat.DJ17:
        subprocess.call(['python', os.path.join(project_root, 'manage.py'), 'makemigrations'], env=env)
        subprocess.call(['python', os.path.join(project_root, 'manage.py'), 'migrate'], env=env)
    else:
        subprocess.call(['python', os.path.join(project_root, 'manage.py'), 'syncdb', '--noinput'], env=env)
    subprocess.call(['python', os.path.join(project_root, 'manage.py'), 'collectstatic', '--noinput'], env=env)
    sys.stdout.write("Please enter the project directory {0}, and run python manage.py createsuperuser and"
                     " python manage.py runserver to start. The admin can be found at localhost:8000/admin. You may also want to set your "
                     "DJANGO_SETTINGS_MODULE environment variable to {0}.settings \n".format(project_name))



import os

from django.core.management import call_command
from django.test import TestCase

from ..models import ScriptVersion

from . import config
from . import mixins


class CommandTests(mixins.ScriptFactoryMixin, TestCase):
    def setUp(self):
        # don't setup scripts, but we want to still tear down after each test
        pass

    def test_addscript(self):
        call_command('addscript', os.path.join(config.WOOEY_TEST_SCRIPTS, 'choices.py'))
        # Test we can update the script
        script_version = ScriptVersion.objects.latest('created_date')
        old_parameters = list(script_version.get_parameters())
        call_command('addscript', '--update', os.path.join(config.WOOEY_TEST_SCRIPTS, 'choices.py'))
        new_version = ScriptVersion.objects.latest('created_date')

        # make sure we updated
        self.assertEqual(new_version.script_iteration, script_version.script_iteration + 1)

        # Make sure the parameters have not changed
        self.assertListEqual(old_parameters, list(new_version.get_parameters()))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0027_parameter_order'),
    ]

    operations = [
        migrations.AlterField(
            model_name='script',
            name='save_path',
            field=models.CharField(blank=True, help_text='By default save to the script name, this will change the output folder.', max_length=255, null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='scriptparameter',
            name='param_help',
            field=models.TextField(verbose_name='help', blank=True, null=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='scriptversion',
            name='script_path',
            field=models.FileField(upload_to=''),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wooeyfile',
            name='filepath',
            field=models.FileField(upload_to='', max_length=500),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wooeyjob',
            name='status',
            field=models.CharField(choices=[('submitted', 'Submitted'), ('running', 'Running'), ('completed', 'Completed'), ('deleted', 'Deleted')], default='submitted', max_length=255),
            preserve_default=True,
        ),
    ]



from __future__ import absolute_import, print_function, unicode_literals
__author__ = 'chris'
import os
import errno
import importlib
import json
import six
import uuid
from io import IOBase

from django.core.files.storage import default_storage
from django.core.files.base import ContentFile
from django.db import models
from django.conf import settings
from django.core.files.storage import SuspiciousFileOperation
from django.core.urlresolvers import reverse_lazy, reverse
from django.contrib.auth.models import Group
from django.utils.translation import ugettext_lazy as _
from django.db import transaction
from django.utils.text import get_valid_filename

from autoslug import AutoSlugField

from celery import states

from .. import settings as wooey_settings
from .. backend import utils
from ..django_compat import get_cache

from . mixins import UpdateScriptsMixin, ModelDiffMixin, WooeyPy2Mixin
from .. import django_compat

# TODO: Handle cases where celery is not setup but specified to be used
tasks = importlib.import_module(wooey_settings.WOOEY_CELERY_TASKS)


class ScriptGroup(UpdateScriptsMixin, WooeyPy2Mixin, models.Model):
    """
        This is a group of scripts, it holds general information
        about a collection of scripts, and allows for custom descriptions

    """
    group_name = models.TextField()
    slug = AutoSlugField(populate_from='group_name', unique=True)
    group_description = models.TextField(null=True, blank=True)
    group_order = models.SmallIntegerField(default=1)
    is_active = models.BooleanField(default=True)
    user_groups = models.ManyToManyField(Group, blank=True)

    class Meta:
        app_label = 'wooey'
        verbose_name = _('script group')
        verbose_name_plural = _('script groups')

    def __str__(self):
        return self.group_name


class Script(ModelDiffMixin, WooeyPy2Mixin, models.Model):
    script_name = models.CharField(max_length=255)
    slug = AutoSlugField(populate_from='script_name', unique=True)
    # we create defaults for the script_group in the clean method of the model. We have to set it to null/blank=True
    # or else we will fail form validation before we hit the model.
    script_group = models.ForeignKey('ScriptGroup', null=True, blank=True)
    script_description = models.TextField(blank=True, null=True)
    documentation = models.TextField(blank=True, null=True)
    script_order = models.PositiveSmallIntegerField(default=1)
    is_active = models.BooleanField(default=True)
    user_groups = models.ManyToManyField(Group, blank=True)

    execute_full_path = models.BooleanField(default=True)  # use full path for subprocess calls
    save_path = models.CharField(max_length=255, blank=True, null=True,
                                 help_text='By default save to the script name,'
                                           ' this will change the output folder.')

    created_date = models.DateTimeField(auto_now_add=True)
    modified_date = models.DateTimeField(auto_now=True)

    class Meta:
        app_label = 'wooey'
        verbose_name = _('script')
        verbose_name_plural = _('scripts')

    def __str__(self):
        return self.script_name

    def get_url(self):
        return reverse('wooey:wooey_script', kwargs={'slug': self.slug})

    @property
    def latest_version(self):
        return self.script_version.get(default_version=True)

    def clean(self):
        if self.script_group is None:
            group = ScriptGroup.objects.filter(group_name=wooey_settings.WOOEY_DEFAULT_SCRIPT_GROUP).order_by('pk').first()
            if not group:
                group, created = ScriptGroup.objects.get_or_create(group_name=wooey_settings.WOOEY_DEFAULT_SCRIPT_GROUP)
            self.script_group = group

    def get_previous_versions(self):
        return self.script_version.all().order_by('script_version', 'script_iteration')


class ScriptVersion(ModelDiffMixin, WooeyPy2Mixin, models.Model):
    # when a script updates, increment this to keep old scripts that are cloned working. The downside is we get redundant
    # parameters, but even a huge site may only have a few thousand parameters to query though.
    script_version = models.CharField(max_length=50, help_text='The script version.', blank=True, default='1')
    script_iteration = models.PositiveSmallIntegerField(default=1)
    script_path = models.FileField() if django_compat.DJANGO_VERSION >= django_compat.DJ17 else models.FileField(upload_to=wooey_settings.WOOEY_SCRIPT_DIR)
    default_version = models.BooleanField(default=False)
    script = models.ForeignKey('Script', related_name='script_version')

    created_date = models.DateTimeField(auto_now_add=True)
    modified_date = models.DateTimeField(auto_now=True)

    class Meta:
        app_label = 'wooey'
        verbose_name = _('script version')
        verbose_name_plural = _('script versions')

    def __str__(self):
        return '{}({}: {})'.format(self.script.script_name, self.script_version, self.script_iteration)

    def get_url(self):
        return reverse('wooey:wooey_script', kwargs={'slug': self.script.slug})

    def get_script_path(self):
        local_storage = utils.get_storage(local=True)
        path = local_storage.path(self.script_path.path)
        return path if self.script.execute_full_path else os.path.split(path)[1]

    def get_parameters(self):
        return ScriptParameter.objects.filter(script_version=self).order_by('param_order', 'pk')


class WooeyJob(WooeyPy2Mixin, models.Model):
    """
    This model serves to link the submitted celery tasks to a script submitted
    """
    # blank=True, null=True is to allow anonymous users to submit jobs
    user = models.ForeignKey(settings.AUTH_USER_MODEL, blank=True, null=True)
    celery_id = models.CharField(max_length=255, null=True)
    uuid = models.CharField(max_length=255, default=uuid.uuid4, unique=True)
    job_name = models.CharField(max_length=255)
    job_description = models.TextField(null=True, blank=True)
    stdout = models.TextField(null=True, blank=True)
    stderr = models.TextField(null=True, blank=True)

    DELETED = 'deleted'
    SUBMITTED = 'submitted'
    COMPLETED = 'completed'
    RUNNING = 'running'

    STATUS_CHOICES = (
        (SUBMITTED, _('Submitted')),
        (RUNNING, _('Running')),
        (COMPLETED, _('Completed')),
        (DELETED, _('Deleted')),
    )

    status = models.CharField(max_length=255, default=SUBMITTED, choices=STATUS_CHOICES)

    save_path = models.CharField(max_length=255, blank=True, null=True)
    command = models.TextField()
    created_date = models.DateTimeField(auto_now_add=True)
    modified_date = models.DateTimeField(auto_now=True)
    script_version = models.ForeignKey('ScriptVersion')

    class Meta:
        app_label = 'wooey'
        verbose_name = _('wooey job')
        verbose_name_plural = _('wooey jobs')

    def __str__(self):
        return self.job_name

    def get_parameters(self):
        return ScriptParameters.objects.filter(job=self).order_by('pk')

    def submit_to_celery(self, **kwargs):
        if kwargs.get('resubmit'):
            params = self.get_parameters()
            user = kwargs.get('user')
            self.pk = None
            self.user = None if user is None or not user.is_authenticated() else user
            # clear the output channels
            self.celery_id = None
            self.uuid = uuid.uuid4()
            self.stdout = ''
            self.stderr = ''
            self.save()
            with transaction.atomic():
                for param in params:
                    param.pk = None
                    param.job = self
                    param.recreate()
                    param.save()
        self.status = self.SUBMITTED
        self.save()
        task_kwargs = {'wooey_job': self.pk, 'rerun': kwargs.pop('rerun', False)}

        if task_kwargs.get('rerun'):
            utils.purge_output(job=self)
        if wooey_settings.WOOEY_CELERY:
            results = tasks.submit_script.delay(**task_kwargs)
        else:
            results = tasks.submit_script(**task_kwargs)
        return self

    def get_resubmit_url(self):
        return reverse('wooey:wooey_script_clone', kwargs={'slug': self.script_version.script.slug, 'job_id': self.pk})

    @property
    def output_path(self):
        return os.path.join(wooey_settings.WOOEY_FILE_DIR,
                            get_valid_filename(self.user.username if self.user is not None else ''),
                            get_valid_filename(self.script_version.script.slug if not self.script_version.script.save_path else self.script_version.script.save_path),
                            str(self.uuid))

    def get_output_path(self):
        path = self.output_path
        utils.mkdirs(os.path.join(settings.MEDIA_ROOT, path))
        return path

    def get_upload_path(self):
        path = self.output_path
        utils.mkdirs(os.path.join(settings.MEDIA_ROOT, path))
        return path

    def get_relative_path(self, path):
        # We make the filename relative to the MEDIA_ROOT, this is for filesystems that can change between
        # machines. We also want to omit any leading path separators so we can join the path to whatever
        # MEDIA_ROOT is currently at work instead of giving a path from a root
        return path[path.find(self.get_output_path()):].lstrip(os.path.sep)

    def get_realtime_key(self):
        return 'wooeyjob_{}_rt'.format(self.pk)

    def update_realtime(self, stdout='', stderr='', delete=False):
        wooey_cache = wooey_settings.WOOEY_REALTIME_CACHE
        if delete is False and wooey_cache is None:
            self.stdout = stdout
            self.stderr = stderr
            self.save()
        elif wooey_cache is not None:
            cache = get_cache(wooey_cache)
            if delete:
                cache.delete(self.get_realtime_key())
            else:
                cache.set(self.get_realtime_key(), json.dumps({'stdout': stdout, 'stderr': stderr}))

    def get_realtime(self):
        wooey_cache = wooey_settings.WOOEY_REALTIME_CACHE
        if wooey_cache is not None:
            cache = get_cache(wooey_cache)
            out = cache.get(self.get_realtime_key())
            if out:
                return json.loads(out)
        return {'stdout': self.stdout, 'stderr': self.stderr}

    def get_stdout(self):
        if self.status != WooeyJob.COMPLETED:
            rt = self.get_realtime().get('stdout')
            if rt:
                return rt
        return self.stdout

    def get_stderr(self):
        if self.status != WooeyJob.COMPLETED:
            rt = self.get_realtime().get('stderr')
            if rt:
                return rt
        return self.stderr


class ScriptParameterGroup(UpdateScriptsMixin, WooeyPy2Mixin, models.Model):
    group_name = models.TextField()
    hidden = models.BooleanField(default=False)
    script_version = models.ForeignKey('ScriptVersion')

    class Meta:
        app_label = 'wooey'
        verbose_name = _('script parameter group')
        verbose_name_plural = _('script parameter groups')

    def __str__(self):
        return '{}: {}'.format(self.script_version.script.script_name, self.group_name)


class ScriptParameter(UpdateScriptsMixin, WooeyPy2Mixin, models.Model):
    """
        This holds the parameter mapping for each script, and enforces uniqueness by each script via a FK.
    """
    script_version = models.ManyToManyField('ScriptVersion')
    short_param = models.CharField(max_length=255, blank=True)
    script_param = models.CharField(max_length=255)
    slug = AutoSlugField(populate_from='script_param', unique=True)
    is_output = models.BooleanField(default=None)
    required = models.BooleanField(default=False)
    choices = models.CharField(max_length=255, null=True, blank=True)
    choice_limit = models.CharField(max_length=10, null=True, blank=True)
    collapse_arguments = models.BooleanField(
        default=True,
        help_text=_('Collapse separate inputs to a given argument to a single input (ie: --arg 1 --arg 2 becomes --arg 1 2)')
    )
    form_field = models.CharField(max_length=255)
    default = models.CharField(max_length=255, null=True, blank=True)
    input_type = models.CharField(max_length=255)
    param_help = models.TextField(verbose_name='help', null=True, blank=True)
    is_checked = models.BooleanField(default=False)
    hidden = models.BooleanField(default=False)
    parameter_group = models.ForeignKey('ScriptParameterGroup')
    param_order = models.SmallIntegerField('The order the parameter appears to the user.', default=0)

    class Meta:
        app_label = 'wooey'
        verbose_name = _('script parameter')
        verbose_name_plural = _('script parameters')

    @property
    def multiple_choice(self):
        choice_limit = json.loads(self.choice_limit)
        if choice_limit is None:
            return False
        try:
            choice_limit = int(choice_limit)
        except ValueError:
            # it's not a set # of choices that is a max, it's either >=0, or >=1, which are the same for a front-end
            # since validation of >=0 or >=1 is performed outside of the form.
            return True
        else:
            return choice_limit > 1

    @property
    def max_choices(self):
        choice_limit = json.loads(self.choice_limit)
        if choice_limit is None:
            return 1
        try:
            choice_limit = int(choice_limit)
        except ValueError:
            # for this, it's either >=0 or >=1 so as many as they want.
            return -1
        else:
            return choice_limit

    def __str__(self):
        scripts = ', '.join([i.script.script_name for i in self.script_version.all()])
        return '{}: {}'.format(scripts, self.script_param)


# TODO: find a better name for this class. Job parameter? SelectedParameter?
class ScriptParameters(WooeyPy2Mixin, models.Model):
    """
        This holds the actual parameters sent with the submission
    """
    # the details of the actual executed scripts
    job = models.ForeignKey('WooeyJob')
    parameter = models.ForeignKey('ScriptParameter')
    # we store a JSON dumped string in here to attempt to keep our types in order
    _value = models.TextField(db_column='value')

    BOOLEAN = 'BooleanField'
    CHAR = 'CharField'
    CHOICE = 'ChoiceField'
    FILE = 'FileField'
    FLOAT = 'FloatField'
    INTEGER = 'IntegerField'

    WOOEY_FIELD_MAP = {
        BOOLEAN: lambda x: str(x).lower() == 'true',
        CHAR: str,
        CHOICE: str,
        FLOAT: float,
        INTEGER: int,
    }

    class Meta:
        app_label = 'wooey'
        verbose_name = _('script parameters')

    def __str__(self):
        try:
            value = self.value
        except IOError:
            value = _('FILE NOT FOUND')
        except SuspiciousFileOperation:
            value = _('File outside of project')
        return '{}: {}'.format(self.parameter.script_param, value)

    def get_subprocess_value(self):
        value = self.value
        if self.value is None:
            return None
        field = self.parameter.form_field
        param = self.parameter.short_param
        com = {'parameter': param, 'script_parameter': self.parameter}
        if field == self.BOOLEAN:
            if value:
                return com
            else:
                del com['parameter']
        if field == self.FILE:
            if self.parameter.is_output:
                try:
                    value = value.path
                except AttributeError:
                    value = utils.get_storage(local=True).path(value)
                    # trim the output path, we don't want to be adding our platform specific paths to the output
                    op = self.job.get_output_path()
                    #TODO : use os.path.sep
                    value = value[value.find(op)+len(op)+1:]
            else:
                # make sure we have it locally otherwise download it
                if not utils.get_storage(local=True).exists(value.path):
                    new_path = utils.get_storage(local=True).save(value.path, value)
                    value = new_path
                else:
                    # return the string for processing
                    value = value.path
        try:
            float(value)
            value = str(value)
        except ValueError:
            pass
        com['value'] = value if isinstance(value, six.string_types) else six.u(value)
        return com

    def force_value(self, value):
        self._value = json.dumps(value)

    def recreate(self):
        # we want to change filefields to reflect whatever is the current job's path. This is currently used for
        # job resubmission
        value = json.loads(self._value)
        if value is not None:
            field = self.parameter.form_field
            if field == self.FILE:
                # we are perfectly fine using old input files instead of recreating them, so only check output files
                if self.parameter.is_output:
                    new_path = self.job.get_output_path()
                    new_root, new_id = os.path.split(new_path)
                    # we want to remove the root + the old job's pk
                    value = value[value.find(new_root)+len(new_root)+1:]
                    value = value[value.find(os.path.sep)+1:]
                    # we want to create a new path for the current job
                    path = os.path.join(new_path, self.parameter.slug if not value else value)
                    value = path
                    self._value = json.dumps(value)

    @property
    def value(self):
        value = json.loads(self._value)
        if value is not None:
            field = self.parameter.form_field
            if field == self.FILE:
                try:
                    file_obj = utils.get_storage_object(value)
                    value = file_obj
                except IOError:
                    # this can occur when the storage object is not yet made for output
                    if self.parameter.is_output:
                        return value
                    raise IOError
        return value

    @value.setter
    def value(self, value):
        # coerce the value to the proper type and store as json to make it persistent as well as have json
        #  handle type conversion on the way back out
        field = self.parameter.form_field
        add_file = False
        checksum = None
        if field == self.CHAR:
            if value is None:
                value = None
            elif field == self.CHAR:
                if not value:
                    value = None
            else:
                value = self.WOOEY_FIELD_MAP[field](value)
        elif field == self.INTEGER:
            value = self.WOOEY_FIELD_MAP[field](value) if isinstance(value, int) or str(value).isdigit() else None
        elif field == self.BOOLEAN:
            if value is None or value is False:
                value = None
            if value:
                value = True
        elif field == self.FILE:
            if self.parameter.is_output:
                # make a fake object for it
                path = os.path.join(self.job.get_output_path(), self.parameter.slug if not value else value)
                value = path
            else:
                if value:
                    local_storage = utils.get_storage(local=True)
                    current_path = local_storage.path(value)
                    checksum = utils.get_checksum(value)
                    path = utils.get_upload_path(current_path, checksum=checksum)
                    if hasattr(value, 'size'):
                        filesize = value.size
                    elif issubclass(type(value), IOBase):
                        value.seek(0, 2)
                        filesize = value.tell()
                        value.seek(0)
                    else:
                        filesize = None
                    if not local_storage.exists(path) or (filesize is not None and local_storage.size(path) != filesize):
                        local_path = local_storage.save(path, value)
                    else:
                        local_path = local_storage.path(path)
                        local_path = os.path.join(os.path.split(path)[0], os.path.split(local_path)[1])
                    remote_storage = utils.get_storage(local=False)
                    if not remote_storage.exists(path) or (filesize is not None and remote_storage.size(path) != filesize):
                        local_path = remote_storage.save(local_path, value)
                    add_file = True
                    value = local_path
        self._value = json.dumps(value)
        if add_file:
            # make a WooeyFile so the user can share it/etc.
            # get the system path for the file
            local_path = utils.get_storage(local=True).path(local_path)
            fileinfo = utils.get_file_info(local_path)
            # save ourself first, we have to do this because we are referenced in WooeyFile
            self.save()
            if checksum is None:
                checksum = utils.get_checksum(local_path)
            wooey_file, file_created = WooeyFile.objects.get_or_create(checksum=checksum)
            if file_created:
                wooey_file.filetype = fileinfo.get('type')
                wooey_file.filepreview = fileinfo.get('preview')
                save_file = utils.get_storage().open(local_path)
                save_path = path
                wooey_file.filepath.save(save_path, save_file, save=False)
                wooey_file.filepath.name = save_path
                wooey_file.save()

            UserFile.objects.get_or_create(job=self.job, system_file=wooey_file,
                                           parameter=self, filename=os.path.split(local_path)[1])


class UserFile(WooeyPy2Mixin, models.Model):
    filename = models.TextField()
    job = models.ForeignKey('WooeyJob')
    system_file = models.ForeignKey('WooeyFile')
    parameter = models.ForeignKey('ScriptParameters', null=True, blank=True)

    class Meta:
        app_label = 'wooey'

    def __str__(self):
        return '{}: {}'.format(self.job.job_name, self.system_file)


class WooeyFile(WooeyPy2Mixin, models.Model):
    filepath = models.FileField(max_length=500) if django_compat.DJANGO_VERSION >= django_compat.DJ17 else models.FileField(max_length=500, upload_to=wooey_settings.WOOEY_SCRIPT_DIR)
    filepreview = models.TextField(null=True, blank=True)
    filetype = models.CharField(max_length=255, null=True, blank=True)
    size_bytes = models.IntegerField(null=True)
    checksum = models.CharField(max_length=40, blank=True)

    class Meta:
        app_label = 'wooey'
        verbose_name = _('wooey file')
        verbose_name_plural = _('wooey files')

    def __str__(self):
        return self.filepath.name



from django.views.generic import DetailView, TemplateView
from django.http import HttpResponse, HttpResponseRedirect, HttpResponseForbidden, HttpResponseNotFound, HttpResponseBadRequest
from django.core.urlresolvers import reverse
from django.conf import settings
from django.views.decorators.csrf import ensure_csrf_cookie
from django.contrib.contenttypes.models import ContentType

from ..django_compat import JsonResponse

from django.contrib.auth.decorators import login_required

from ..models import Favorite


@login_required
@ensure_csrf_cookie
def toggle_favorite(request):
    """
    Add/remove an object to the user's favorites. Checks for existance and adds if not, else removes.
    This is the underlying mechanism for adding items to the user 'scrapbook' and favorite scripts.

    :param request:
    :param file_id:
    :return:
    """
    if not request.is_ajax():
        return HttpResponseForbidden()

    try:
        app, model, pk = request.POST['app'], request.POST['model'], int(request.POST['pk'])

    except ValueError:
        return HttpResponseBadRequest()

    try:
        ctype = ContentType.objects.get(app_label=app, model=model)
        obj = ctype.get_object_for_this_type(id=pk)

    except Favorite.DoesNotExist:
        return HttpResponseNotFound()

    try:
        fave = Favorite.objects.get(content_type=ctype, object_id=obj.id, user=request.user)

    except Favorite.DoesNotExist:
        # Does not exist, so create it
        fave = Favorite(content_object=obj, user=request.user)
        fave.save()
        is_favorite = True

    else:
        # Exists, so delete it
        fave.delete()
        is_favorite = False

    # Return the current total number for UI updates
    favorites_count = Favorite.objects.filter(content_type=ctype, user=request.user).count()

    return JsonResponse({
        'app': app,
        'model': model,
        'pk': pk,
        'is_favorite': is_favorite,
        'count': favorites_count,
    })



# TODO: Test for viewing a user's job as an anonymous user (fail case)

import json

from django.test import TestCase, RequestFactory, Client
from django.core.urlresolvers import reverse
from django.contrib.auth.models import AnonymousUser
from django.http import Http404

from nose.tools import raises

from . import factories, mixins, config
from ..views import wooey_celery
from .. import views as wooey_views
from .. import settings


def load_JSON_dict(d):
    return json.loads(d.decode('utf-8'))


class CeleryViews(mixins.ScriptFactoryMixin, mixins.FileCleanupMixin, TestCase):
    def setUp(self):
        super(CeleryViews, self).setUp()
        self.factory = RequestFactory()
        # the test server doesn't have celery running
        settings.WOOEY_CELERY = False

    def test_celery_results(self):
        request = self.factory.get(reverse('wooey:all_queues_json'))
        user = factories.UserFactory()
        request.user = user
        response = wooey_celery.all_queues_json(request)
        d = response.content.decode("utf-8")
        self.assertEqual({u'items': {u'global': [], u'results': [], u'user': []},
                          u'totals': {u'global': 0, u'results': 0, u'user': 0}
                            }, json.loads(d))

        job = factories.generate_job(self.translate_script)
        job.save()
        response = wooey_celery.all_queues_json(request)
        d = json.loads(response.content.decode("utf-8"))
        self.assertEqual(1, d['totals']['global'])

        job.user = user
        job.save()
        response = wooey_celery.all_queues_json(request)
        d = json.loads(response.content.decode("utf-8"))
        # we now are logged in, make sure the job appears under the user jobs
        self.assertEqual(1, d['totals']['user'])

        user = AnonymousUser()
        request.user = user
        response = wooey_celery.all_queues_json(request)
        d = json.loads(response.content.decode("utf-8"))
        # test empty response since anonymous users should not see users jobs
        self.assertEqual(d['items']['results'], [])
        self.assertEqual(d['items']['user'], [])

    def test_celery_commands(self):
        user = factories.UserFactory()
        job = factories.generate_job(self.translate_script)
        job.user = user
        job.save()
        celery_command = {'celery-command': ['delete'], 'job-id': [job.pk]}
        # test that we cannot modify a users script
        request = self.factory.post(reverse('wooey:celery_task_command'),
                                    celery_command)
        anon = AnonymousUser()
        request.user = anon
        response = wooey_celery.celery_task_command(request)
        d = response.content.decode("utf-8")
        self.assertFalse(json.loads(d).get('valid'))

        # test a nonsense command
        celery_command.update({'celery-command': ['thisshouldfail']})
        response = wooey_celery.celery_task_command(request)
        d = response.content.decode("utf-8")
        self.assertFalse(json.loads(d).get('valid'))

        # test that the user can interact with it
        # the stop command will break, so currently untested here until I figure it out
        for i in ['resubmit', 'rerun', 'clone', 'delete']:
            celery_command.update({'celery-command': [i]})
            request = self.factory.post(reverse('wooey:celery_task_command'),
                                    celery_command)
            request.user = user
            response = wooey_celery.celery_task_command(request)
            d = response.content.decode("utf-8")
            self.assertTrue(json.loads(d).get('valid'))

    def test_celery_task_view(self):
        user = factories.UserFactory()
        job = factories.generate_job(self.translate_script)
        job.user = user
        job.save()

        # test that an anonymous user cannot view a user's job
        view = wooey_celery.JobView.as_view()
        request = self.factory.get(reverse('wooey:celery_results', kwargs={'job_id': job.pk}))
        request.user = AnonymousUser()
        response = view(request, job_id=job.pk)
        self.assertIn('job_error', response.context_data)
        self.assertNotIn('job_info', response.context_data)

        # test the user can view the job
        request.user = user
        response = view(request, job_id=job.pk)
        self.assertNotIn('job_error', response.context_data)
        self.assertIn('job_info', response.context_data)

    @raises(Http404)
    def test_celery_nonexistent_task(self):
        # test request for non-existent job, should raise 404
        view = wooey_celery.JobView.as_view()
        request = self.factory.get(reverse('wooey:celery_results', kwargs={'job_id': '-1'}))
        response = view(request, job_id=-1)


class WooeyViews(mixins.ScriptFactoryMixin, mixins.FileCleanupMixin, TestCase):
    def setUp(self):
        super(WooeyViews, self).setUp()
        self.factory = RequestFactory()
        self.script_view_func = wooey_views.WooeyScriptJSON.as_view()
        # the test server doesn't have celery running
        settings.WOOEY_CELERY = False

    def test_multiple_choice_clone(self):
        from ..backend import utils
        script_version = self.choice_script
        script = script_version.script
        choices = ['2', '1', '3']
        choice_param = 'two_choices'
        job = utils.create_wooey_job(script_version_pk=script_version.pk, data={'job_name': 'abc', choice_param: choices, 'wooey_type': script_version.pk})
        request = self.factory.post(reverse('wooey:wooey_script_clone',
                                           kwargs={'slug': job.script_version.script.slug, 'job_id': job.pk}),
                                    data={'wooey_type': script_version.pk})
        request.user = AnonymousUser()
        response = self.script_view_func(request, pk=job.pk, job_id=job.pk)
        self.assertEqual(response.status_code, 200)

    def test_multiple_choice(self):
        user = factories.UserFactory()
        script_version = self.choice_script
        script = script_version.script
        url = reverse('wooey:wooey_script', kwargs={'slug': script.slug})
        data = {'job_name': 'abc', 'wooey_type': script_version.pk, 'two_choices': ['2', '1', '3']}
        filecount = 0
        for i, v in config.SCRIPT_DATA['choices']['files'].items():
            data[i] = v
            filecount += len(v)
        request = self.factory.post(url, data=data)
        request.user = user
        response = self.script_view_func(request)
        d = load_JSON_dict(response.content)
        self.assertTrue(d['valid'], d)
        self.assertEqual(sum([len(request.FILES.getlist(i)) for i in request.FILES.keys()]), filecount)

        # test submitting this in the 'currently' field
        from ..models import WooeyJob
        job = WooeyJob.objects.latest('created_date')
        files = [i.value.name for i in job.get_parameters() if i.parameter.slug == 'multiple_file_choices']

        data['multiple_file_choices'] = files
        request = self.factory.post(url, data=data)
        request.user = user
        response = self.script_view_func(request)
        self.assertEqual(response.status_code, 200)
        d = load_JSON_dict(response.content)
        self.assertTrue(d['valid'], d)

        # check the files are actually with the new model
        job = WooeyJob.objects.latest('created_date')
        new_files = [i.value.url for i in job.get_parameters() if i.parameter.slug == 'multiple_file_choices']
        self.assertEqual(len(new_files), len(files))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import wooey.models.mixins


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0008_short_param_admin'),
    ]

    operations = [
        migrations.CreateModel(
            name='ScriptVersion',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('script_version', models.CharField(default='1', help_text='The script version.', max_length=50, blank=True)),
                ('script_iteration', models.PositiveSmallIntegerField(default=1)),
                ('script_path', models.FileField(upload_to=b'')),
                ('default_version', models.BooleanField(default=False)),
                ('created_date', models.DateTimeField(auto_now_add=True)),
                ('modified_date', models.DateTimeField(auto_now=True)),
                ('script', models.ForeignKey(related_name='script_version_new', to='wooey.Script')),
            ],
            bases=(wooey.models.mixins.ModelDiffMixin, wooey.models.mixins.WooeyPy2Mixin, models.Model),
        ),
        migrations.AddField(
            model_name='scriptparameter',
            name='script_version',
            field=models.ForeignKey(null=True, to='wooey.ScriptVersion'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='scriptparametergroup',
            name='script_version',
            field=models.ForeignKey(null=True, to='wooey.ScriptVersion'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='wooeyjob',
            name='script_version',
            field=models.ForeignKey(null=True, to='wooey.ScriptVersion'),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-07-02 14:27
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('wooey', '0024_script_parameter_m2m_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='scriptparameter',
            name='script_version',
        ),
    ]



import os
import uuid

from django.test import TestCase, Client

from . import factories, config, mixins
from .. import version


class ScriptTestCase(mixins.ScriptFactoryMixin, TestCase):

    def test_multiple_choices(self):
        # load our choice script
        script = self.choice_script

        multiple_choice_param = 'two_choices'
        single_choice_param = 'one_choice'
        optional_choice_param = 'all_choices'
        # test that we are a multiple choice entry
        from ..models import ScriptParameter
        param = ScriptParameter.objects.get(slug=multiple_choice_param)
        self.assertTrue(param.multiple_choice)

        # test our limit
        self.assertEqual(param.max_choices, 2)

        # test with a singular case
        param = ScriptParameter.objects.get(slug=single_choice_param)
        self.assertFalse(param.multiple_choice)
        self.assertEqual(param.max_choices, 1)

        # test cases that have variable requirements
        param = ScriptParameter.objects.get(slug=optional_choice_param)
        self.assertTrue(param.multiple_choice)
        self.assertEqual(param.max_choices, -1)


class ScriptGroupTestCase(TestCase):

    def test_script_group_creation(self):
        group = factories.ScriptGroupFactory()


class TestJob(mixins.ScriptFactoryMixin, mixins.FileCleanupMixin, mixins.FileMixin, TestCase):
    urls = 'wooey.test_urls'

    def get_local_url(self, fileinfo):
        from ..backend import utils
        local_storage = utils.get_storage(local=True)
        return local_storage.url(fileinfo['object'].filepath.name)

    def test_jobs(self):
        script = self.translate_script
        from ..backend import utils
        job = utils.create_wooey_job(script_version_pk=script.pk, data={'job_name': 'abc', 'sequence': 'aaa', 'out': 'abc'})
        job = job.submit_to_celery()
        old_pk = job.pk
        new_job = job.submit_to_celery(resubmit=True)
        self.assertNotEqual(old_pk, new_job.pk)
        # test rerunning, our output should be removed
        from ..models import UserFile
        old_output = sorted([i.pk for i in UserFile.objects.filter(job=new_job)])
        # the pk will not change here since we are using rerun=True
        new_job.submit_to_celery(rerun=True)
        # check that we overwrite our output
        new_output = sorted([i.pk for i in UserFile.objects.filter(job=new_job)])
        # Django 1.6 has a bug where they are reusing pk numbers
        if version.DJANGO_VERSION >= version.DJ17:
            self.assertNotEqual(old_output, new_output)
        self.assertEqual(len(old_output), len(new_output))
        # check the old entries are gone
        if version.DJANGO_VERSION >= version.DJ17:
            # Django 1.6 has a bug where they are reusing pk numbers, so once again we cannot use this check
            self.assertEqual([], list(UserFile.objects.filter(pk__in=old_output)))

        file_previews = utils.get_file_previews(job)
        for group, files in file_previews.items():
            for fileinfo in files:
                # for testing, we use the local url
                response = Client().get(self.get_local_url(fileinfo))
                self.assertEqual(response.status_code, 200)

        # check our download links are ok
        # upload the file first to our storage engine so this works in tests
        local_storage = utils.get_storage(local=True)
        fasta_path = local_storage.save('fasta.fasta', open(os.path.join(config.WOOEY_TEST_DATA, 'fasta.fasta')))
        fasta_file = local_storage.open(fasta_path)
        job = utils.create_wooey_job(script_version_pk=script.pk,
                                        data={
                                            'fasta': fasta_file,
                                            'out': 'abc',
                                            'job_name': 'abc'
                                        }
                                     )

        # check our upload link is ok
        file_previews = utils.get_file_previews(job)
        for group, files in file_previews.items():
            for fileinfo in files:
                response = Client().get(self.get_local_url(fileinfo))
                self.assertEqual(response.status_code, 200)

    def test_file_sharing(self):
        # this tests whether a file uploaded by one job will be referenced by a second job instead of being duplicated
        # on the file system
        new_file = self.storage.open(self.get_any_file())
        script_slug = 'multiple_file_choices'
        script = self.choice_script
        from ..backend import utils
        job = utils.create_wooey_job(script_version_pk=script.pk, data={'job_name': 'job1', script_slug: new_file})
        job = job.submit_to_celery()
        job2 = utils.create_wooey_job(script_version_pk=script.pk, data={'job_name': 'job2', script_slug: new_file})
        job2 = job2.submit_to_celery()
        from ..models import UserFile
        job1_files = [i for i in UserFile.objects.filter(job=job, parameter__isnull=False) if i.parameter.parameter.slug == script_slug]
        job1_file = job1_files[0]
        job2_files = [i for i in UserFile.objects.filter(job=job2, parameter__isnull=False) if i.parameter.parameter.slug == script_slug]
        job2_file = job2_files[0]
        self.assertNotEqual(job1_file.pk, job2_file.pk)
        self.assertEqual(job1_file.system_file, job2_file.system_file)


    def test_multiplechoices(self):
        script = self.choice_script
        choices = ['2', '1', '3']
        choice_param = 'two_choices'

        from ..backend import utils
        job = utils.create_wooey_job(script_version_pk=script.pk, data={'job_name': 'abc', choice_param: choices})
        # make sure we have our choices in the parameters
        choice_params = [i.value for i in job.get_parameters() if i.parameter.slug == choice_param]
        self.assertEqual(choices, choice_params)
        job = job.submit_to_celery()



from os import environ

from django.conf.urls import include, url
from django.conf import settings
from django.conf.urls.static import static

from .wooey_urls import *

if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
    urlpatterns += static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)



from datetime import timedelta

from django.test import TestCase

from . import mixins, factories
from .. import settings as wooey_settings


class TaskTests(mixins.ScriptFactoryMixin, TestCase):

    def test_job_cleanup(self):
        from ..models import WooeyJob
        from ..tasks import cleanup_wooey_jobs
        import time
        anon_job = factories.generate_job(self.translate_script)
        user_job = factories.generate_job(self.translate_script)
        user = factories.UserFactory()
        user_job.user = user
        user_job.save()
        wooey_settings.WOOEY_JOB_EXPIRATION.update({
            'user': timedelta(hours=1),
            'anonymous': timedelta(hours=1),
        })
        cleanup_wooey_jobs()
        self.assertListEqual(list(WooeyJob.objects.all()), [anon_job, user_job])
        time.sleep(0.1)
        wooey_settings.WOOEY_JOB_EXPIRATION.update({
            'user': timedelta(hours=1),
            'anonymous': timedelta(microseconds=1),
        })
        cleanup_wooey_jobs()
        self.assertListEqual(list(WooeyJob.objects.all()), [user_job])

        wooey_settings.WOOEY_JOB_EXPIRATION.update({
            'user': timedelta(microseconds=1),
            'anonymous': timedelta(microseconds=1),
        })

        cleanup_wooey_jobs()
        self.assertListEqual(list(WooeyJob.objects.all()), [])



import os

from django.test import TestCase
from django.conf import settings

from . import config, mixins
from ..backend import utils
from ..models import ScriptVersion
from .. import settings as wooey_settings


class ScriptAdditionTests(mixins.ScriptFactoryMixin, mixins.FileMixin, TestCase):

    def test_command_order(self):
        script = os.path.join(config.WOOEY_TEST_SCRIPTS, 'command_order.py')
        with open(script) as o:
            new_file = self.storage.save(self.filename_func('command_order.py'), o)
        res = utils.add_wooey_script(script_path=new_file, group=None)
        self.assertEqual(res['valid'], True, res['errors'])
        job = utils.create_wooey_job(script_version_pk=res['script'].pk, data={'job_name': 'abc', 'link': 'alink', 'name': 'aname'})
        # These are positional arguments -- we DO NOT want them returning anything
        self.assertEqual(['', ''], [i.parameter.short_param for i in job.get_parameters()])
        # These are the params shown to the user, we want them returning their destination
        # This also checks that we maintain the expected order
        self.assertEqual(['link', 'name'], [i.parameter.script_param for i in job.get_parameters()])
        # Check the job command
        commands = utils.get_job_commands(job=job)[2:]
        self.assertEqual(['alink', 'aname'], commands)

    def test_collapse_arguments(self):
        job = utils.create_wooey_job(script_version_pk=self.choice_script.pk, data={'job_name': 'abc', 'need_at_least_one_numbers': [1,2]})
        commands = utils.get_job_commands(job=job)[2:]
        self.assertEqual(commands, ['--need-at-least-one-numbers', '1', '2'])
        job = utils.create_wooey_job(script_version_pk=self.choice_script.pk,
                                     data={'job_name': 'abc', 'choices_str': [1, 2, 3]})
        commands = utils.get_job_commands(job=job)[2:]
        self.assertEqual(commands, ['--choices-str', '1', '--choices-str', '2', '--choices-str', '3'])

    def test_script_upgrade(self):
        script_path = os.path.join(config.WOOEY_TEST_SCRIPTS, 'command_order.py')
        with open(script_path) as o:
            new_file = self.storage.save(self.filename_func('command_order.py'), o)
        res = utils.add_wooey_script(script_path=new_file, group=None)
        self.assertEqual(res['valid'], True, res['errors'])
        # upgrade script
        script = ScriptVersion.objects.get(pk=res['script'].pk)
        with open(script_path) as o:
            new_script = self.storage.save(self.filename_func('command_order.py'), o)
        script.script_path = new_script
        # we are going to be cloning this, so we lose the old object
        old_pk, old_iter = script.pk, script.script_iteration
        script.save()
        self.assertNotEqual(old_pk, script.pk)
        self.assertNotEqual(old_iter, script.script_iteration)
        # asset we are using the latest script in the frontend
        self.assertIn(script, [i.latest_version for i in utils.get_current_scripts()])
        old_script = ScriptVersion.objects.get(pk=old_pk)
        self.assertNotIn(old_script, [i.latest_version for i in utils.get_current_scripts()])

    def test_script_parameter_upgrade(self):
        script_path = os.path.join(config.WOOEY_TEST_SCRIPTS, 'choices.py')
        script_2_path = os.path.join(config.WOOEY_TEST_SCRIPTS, 'choices_2.py')
        with open(script_path) as o:
            new_file = self.storage.save(self.filename_func('choices.py'), o)
        res = utils.add_wooey_script(script_path=new_file, group=None)
        self.assertEqual(res['valid'], True, res['errors'])
        # upgrade script
        script = ScriptVersion.objects.get(pk=res['script'].pk)
        with open(script_2_path) as o:
            new_script = self.storage.save(self.filename_func('choices.py'), o)
        script.script_path = new_script
        # we are going to be cloning this, so we lose the old object
        old_pk, old_iter = script.pk, script.script_iteration
        script.save()
        self.assertNotEqual(old_pk, script.pk)
        self.assertNotEqual(old_iter, script.script_iteration)
        # asset we are using the latest script in the frontend
        self.assertIn(script, [i.latest_version for i in utils.get_current_scripts()])
        old_script = ScriptVersion.objects.get(pk=old_pk)
        self.assertNotIn(old_script, [i.latest_version for i in utils.get_current_scripts()])
        # Assert that the 'one_choice' parameter is different between the versions, but all else is the same
        old_parameters = list(old_script.get_parameters())
        new_parameters = list(script.get_parameters())
        self.assertListEqual(old_parameters[1:], new_parameters[1:])
        self.assertTrue(new_parameters[0].short_param, '--one-choice-added')
        self.assertTrue(old_parameters[0].short_param, '--one-choice')
